#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Jesus Pena-Izquierdo (jesus.pena@bsc.es)

import os
import pandas as pd
import numpy as np

from bokeh.plotting import figure
from bokeh.io import save, output_file
from bokeh.models import ColumnDataSource, HoverTool

"""
Class: Bokeh plots
---
- General plot functions for histograms 
- Statistical formulas needed to produce the plots
"""
class bokehPlot:

    def __init__(self, csvs, csvh, outd, xl):
        self.csv_hist = csvh
        self.csv_stat = csvs
        self.outdir = outd
        self.xaxis_label = xl
        self.outfile_hist = os.path.join(outd, os.path.basename(csvh).replace('.csv','.html'))
        self.regions = {
          'total' : '-90 to 90 deg latitude', 
          'N_polar' : '60 to 90 deg latitude', 
          'N_midlat' : '30 to 60 deg latitude', 
          'N_hadley' : '0 to 30 deg latitude', 
          'S_hadley' : '-30 to 0 deg latitude', 
          'S_midlat' : '-60 to -30 deg latitude', 
          'S_polar' : '-90 to -60 deg latitude'
        }

    """
    Adjusted boxplot
    """
    def adj_boxplot(self,q1,q3,skew, dev, Ks, Ke):
        IQD = q3 - q1
        if IQD == 0:
            IQD = dev
        return q1 - Ke*IQD*np.exp(-Ks*skew1), q3 + Ke*IQD*np.exp(Ks*skew1)

    def std_boxplot(self,m, dev, K):
        return m -dev*K, m + dev*K

    def iqd_boxplot(self,q1,q3,K = 1.5):
        IQD = q3 - q1
        if IQD == 0:
            IQD = dev
        return q1 - K*IQD, q3 + K*IQD

    def hist2array(self,xm, n):
        x = [[ xi]*int(ni) for (xi, ni) in zip(xm, n)]
        return np.hstack(x)

    def hist2array_cut(self,xm, n, rcut = 0.001):
        Nt = np.sum(n)
        ncum = np.cumsum(n/Nt)
        i_nr0i = np.argwhere( ncum <= rcut ).flatten()
        i_nr0d = np.argwhere( ncum >= (1-rcut)).flatten()
        i_nr0 = np.concatenate((i_nr0i, i_nr0d))
        i_nr1 = np.argwhere((ncum > rcut) & (ncum < 1-rcut)).flatten()
        n_r0 = n[i_nr0]
        xm_r0 = xm[i_nr0]
        max_r0id = [n[i_nr0i[-1]], n[i_nr0d[0]]]
        if 0 in max_r0id:
            max_r0id.remove(0)
        k_r0 = np.min(max_r0id)
        print(k_r0)
        n_r1 = np.round(n[i_nr1]/k_r0)
        xm_r1 = xm[i_nr1]
        x_k0 = hist2array(xm_r0, n_r0)
        x_k1 = hist2array(xm_r1, n_r1)
        return x_k0, x_k1

    """
    Plot histogram 
    """
    def plot_histogram(self, tag='total', col='File'):

        # Read histograms into pandas
        df_hist = pd.read_csv(self.csv_hist)

        # Get bins (left and right) + remove first element
        l = np.delete(df_hist.columns.values, 0).astype(float) 
        xdist = abs(l[0]-l[1])
        r = [ l+xdist for v in l ]
        
        # Get count + remove first element
        y = np.delete(df_hist.loc[df_hist[col] == tag].values, 0).astype(float)

        # Get non-zero bins
        nz = np.nonzero(y)

        # Set up the figure same as before
        p = figure(plot_width = 1100, 
                   plot_height = 600,
                   title = 'Observed values ({})'.format(self.regions[tag]),
                   x_axis_label = self.xaxis_label, 
                   y_axis_label = 'Count')

        # Histograms and points where we have values
        p.vbar(x=l[nz], width=xdist, bottom=0, top=y[nz], color="firebrick")
        p.cross(x=l[nz], y=0, color="red")

        # Save to file
        outfile = self.outfile_hist.replace('.html', '_{}.html'.format(tag))
        print('INFO: Saving plot in {}'.format(outfile))
        output_file(outfile)
        save(p)

